#
# default.py
#
# Copyright (C) 2010 Tim Wentzlau
#
# This file is part of the XBMC Weather Station script
# 
# The Weather Station script is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# XBMC Weather station script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with EventGhost; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import os,xbmc, xbmcgui,threading

# Script constants
__scriptname__ = "Weather stationx"
__author__ = "Tim Wentzlau"
__url__ = "http://www.lavrsen.dk/foswiki/bin/view/Open2300/XBMCWeatherStation"
__credits__ = "Tim Wentzlau"
__version__ = "1.0"

xbmc.log( "[SCRIPT] '%s: version %s' initialized!" % ( __scriptname__, __version__, ), xbmc.LOGNOTICE )

Root = os.getcwd().replace(";","")+"\\"
LIB_DIR = Root+"lib\\"
sys.path.append(LIB_DIR)
STATION_DIR=LIB_DIR+"stations\\"
sys.path.append(STATION_DIR)
RESOURCE_DIR = Root+"resources\\"
sys.path.append(LIB_DIR)

import pngGraph
from skimpyGimpy import canvas
import lacrosse
station=lacrosse.LacrosseStations(24)
 
TEMPOUT=0
TEMPIN=1
WINDCHILL=2
DEWPOINT=3
WINDSPEED=4
WINDDIRECTION=5
PRESSURE=6
HUMIDITY=7
RAINTOTAL=8
RAINHOUR=9
RAINPERIOD=10
MAXFIELDS=11
MAX=0
MIN=1
VALUE=2
DATA=3
 
_ = xbmc.Language( os.getcwd() ).getLocalizedString 
 
#get actioncodes from keymap.xml
ACTION_PREVIOUS_MENU = 10
 
class MyClass(xbmcgui.Window):
	def __init__(self):
		self.imageCount=1
		self.imageWidth=900
		self.imageHeight=(self.getHeight()-90)/5
		self.cancel=False
		
		x=self.getWidth()/2-self.imageWidth/2
		if x<0:
			x=5
		
		y=10
		self.tempOutGraph=xbmcgui.ControlImage(x,y,self.imageWidth,self.imageHeight,"")
		self.addControl(self.tempOutGraph)

		y+=self.imageHeight+10
		self.windSpeedGraph=xbmcgui.ControlImage(x,y,self.imageWidth,self.imageHeight,"")
		self.addControl(self.windSpeedGraph)
		
		y+=self.imageHeight+10
		self.pressureGraph=xbmcgui.ControlImage(x,y,self.imageWidth,self.imageHeight,"")
		self.addControl(self.pressureGraph)

		y+=self.imageHeight+10
		self.humidityGraph=xbmcgui.ControlImage(x,y,self.imageWidth,self.imageHeight,"")
		self.addControl(self.humidityGraph)

		y+=self.imageHeight+10
		self.rainGraph=xbmcgui.ControlImage(x,y,self.imageWidth,self.imageHeight,"")
		self.addControl(self.rainGraph)
		
		y+=self.imageHeight+10
		self.button0 = xbmcgui.ControlButton(x, y, 80, 30, _(30002))
		self.addControl(self.button0)
		self.setFocus(self.button0)

		self.strAction = xbmcgui.ControlLabel(x+self.imageWidth-200, y, 200, 200, '', 'font14', '0xFF00FF00')
		self.addControl(self.strAction)
		self.strAction.setLabel('')

		self.t = threading.Timer(1.0, self.setValues,[True])
		self.t.start()
		
	def setValues(self,showProgress):
		if self.cancel:
			return
		
		
		if self.imageCount>1:
			os.remove(self.tempOutPath)
			os.remove(self.windSpeedPath)
			os.remove(self.pressurePath)
			os.remove(self.humidityPath)
			os.remove(self.rainPath)
		
		self.tempOutPath=Root+"tmp\\ws-tempOut"+str(self.imageCount)+".png"
		self.windSpeedPath=Root+"tmp\\ws-windSpeed"+str(self.imageCount)+".png"
		self.pressurePath=Root+"tmp\\ws-pressure"+str(self.imageCount)+".png"
		self.humidityPath=Root+"tmp\\ws-humidity"+str(self.imageCount)+".png"
		self.rainPath=Root+"tmp\\ws-rain"+str(self.imageCount)+".png"
		
		self.imageCount+=1
		
		self.strAction.setLabel(_(30001))
		updateImages=True
		progressStep=100/6
		progress=0
		if updateImages:
			

			if showProgress:
				dp = xbmcgui.DialogProgress()
				dp.create(_(0),_(30013),"")
				dp.update(0,_(30001))
			
			station.loadData()
			progress+=progressStep
			if showProgress:
				dp.update(progress,_(30014))
			tempGraph=pngGraph.PNGGraph(self.imageWidth,self.imageHeight)
			tempGraph.setTitle(_(30003))
			tempGraph.addDataSet(station.getDataSet(TEMPOUT))
			tempGraph.addDataSet(station.getDataSet(DEWPOINT))
			tempGraph.addDataSet(station.getDataSet(WINDCHILL))
			
			self.paintMinMax(tempGraph.getCanvas(),station.getValue(TEMPOUT),station.getMin(TEMPOUT),station.getMax(TEMPOUT))
			self.paintMinMax(tempGraph.getCanvas(),station.getValue(DEWPOINT),station.getMin(DEWPOINT),station.getMax(DEWPOINT),True,_(30004))
			tempGraph.create(self.tempOutPath)
			
			if self.cancel:
				return
			progress+=progressStep	
			if showProgress:
				dp.update(progress,_(30015))
			
			speedGraph=pngGraph.PNGGraph(self.imageWidth,self.imageHeight)
			speedGraph.setTitle(_(30005))
			speedGraph.addDataSet(station.getDataSet(WINDSPEED))
			self.createWindDirectionImage(speedGraph.getCanvas(),station.getDataSet(WINDDIRECTION),speedGraph.getOffsetX(),speedGraph.getOffsetY()+speedGraph.getGraphHeight()/2,speedGraph.getXTick(),station.getValue(WINDDIRECTION))
			self.paintMinMax(speedGraph.getCanvas(),station.getValue(WINDSPEED),station.getMin(WINDSPEED),station.getMax(WINDSPEED))
			speedGraph.create(self.windSpeedPath)
			
			if self.cancel:
				return
			progress+=progressStep
			if showProgress:
				dp.update(progress,_(30016))
			pressureGraph=pngGraph.PNGGraph(self.imageWidth,self.imageHeight)
			pressureGraph.setTitle(_(30012))
			pressureGraph.addDataSet(station.getDataSet(PRESSURE))
			self.paintMinMax(pressureGraph.getCanvas(),station.getValue(PRESSURE),station.getMin(PRESSURE),station.getMax(PRESSURE))
			self.paintMinMax(pressureGraph.getCanvas(),station.getValue(WINDCHILL),station.getMin(WINDCHILL),station.getMax(WINDCHILL),True,_(30008),[0xff,0xff,0])
			pressureGraph.create(self.pressurePath)
			
			if self.cancel:
				return
			progress+=progressStep
			if showProgress:
				dp.update(progress,_(30017))
			humidityGraph=pngGraph.PNGGraph(self.imageWidth,self.imageHeight)
			humidityGraph.setTitle(_(30006))
			humidityGraph.addDataSet(station.getDataSet(HUMIDITY))
			self.paintMinMax(humidityGraph.getCanvas(),station.getValue(HUMIDITY),station.getMin(HUMIDITY),station.getMax(HUMIDITY))
			humidityGraph.create(self.humidityPath)
			
			progress+=progressStep
			if showProgress:
				dp.update(progress,_(30018))
			rainGraph=pngGraph.PNGGraph(self.imageWidth,self.imageHeight)
			rainGraph.setTitle(_(30007))
			rainGraph.addDataSet(station.getDataSet(RAINHOUR))
			rainGraph.addDataSet(station.getDataSet(RAINPERIOD))
			
			self.paintMinMax(rainGraph.getCanvas(),station.getValue(RAINHOUR),station.getValue(RAINPERIOD),station.getValue(RAINTOTAL),minLabel="  24H",maxLabel="total")
			rainGraph.create(self.rainPath)
			
			
			if showProgress:
				dp.update(100)
				dp.close()
		
		self.tempOutGraph.setImage(self.tempOutPath)
		self.windSpeedGraph.setImage(self.windSpeedPath)
		self.pressureGraph.setImage(self.pressurePath)
		self.humidityGraph.setImage(self.humidityPath)
		self.rainGraph.setImage(self.rainPath)
		
		
		self.strAction.setLabel('')
		
		if (not self.cancel):
			self.t = threading.Timer(10.0, self.setValues,[False])
			self.t.start()
		
		#self.windDirection.setImage(windDirectionPath)

	def paintMinMax(self,canvas,value,min,max,right=False,rightText="",rightColor=[0xff,0,0],minLabel="",maxLabel=""):
		if len(minLabel)==0:
			minLabel=_(30010)
		if len(maxLabel)==0:
			maxLabel=_(30011)
		canvas.saveState()
		canvas.setFont("propell", 1)
		
		if right:
			x=self.imageWidth-150
		else:
			x=30
		
		y=self.imageHeight-15
		if len(rightText)>0:
			canvas.saveState()
			y+=-5
			canvas.setColor(rightColor[0],rightColor[1],rightColor[2])
			canvas.setFont("propell", 1.0)
			canvas.addText(x,y,rightText)
			canvas.restoreState()
			y-=30
		else:
			y-=30
			
		canvas.addText(x,y,str(value))
		canvas.setFont("propell", .8)
		y-=20
		canvas.addText(x,y,maxLabel+": "+str(max))
		y-=20
		canvas.addText(x,y,minLabel+": "+str(min))
		canvas.restoreState()
		
	def createWindDirectionImage(self,canvas,data,x,y,space,value):
		from math import sin,cos,radians
		canvas.saveState()
		left=x
		canvas.setCap(0)
		canvas.setWidth(0.1)
	
		last=0
		for d in data:
			r=radians(d[1])
			cs=cos(r)
			s=sin(r)
			rx=0*cs-(10)*s
			ry=0*s+(10)*cs
			canvas.setColor(0,0xff,0)
			canvas.addLine((left,y),(left+rx,y+ry))
			canvas.setColor(0,0xff,0)
			canvas.addCircle(left,y,3)
			last=r
			left+=space
			
		cs=cos(value)
		s=sin(value)
		rx=0*cs-(30)*s
		ry=0*s+(30)*cs
		canvas.setColor(0,0xff,0)
		left=+800
		canvas.setCap(2)
		canvas.setWidth(2)
		canvas.addLine((left,y),(left+rx,y+ry))
		canvas.setColor(0,0xff,0)
		canvas.addCircle(left,y,10)
		canvas.restoreState()
		
	def onAction(self, action):
		if action == ACTION_PREVIOUS_MENU:
			self.close()
 
	
	def onControl(self, control):
		if control == self.button0:
			self.close()
		
    
	def message(self, message):
		dialog = xbmcgui.Dialog()
		dialog.ok(" My message title", message)
	def stopUpdate(self):
		self.cancel=True
		self.t.cancel();
		
mydisplay = MyClass()
mydisplay .doModal()
mydisplay.stopUpdate()
del mydisplay